@@ -4,30 +4,30 @@ It does this via the Public Suffix List (PSL).
 
     >>> import tldextract
 
-    >>> tldextract.extract('http://forums.news.cnn.com/')
+    >>> tldextract.extract("http://forums.news.cnn.com/")
     ExtractResult(subdomain='forums.news', domain='cnn', suffix='com', is_private=False)
 
-    >>> tldextract.extract('http://forums.bbc.co.uk/') # United Kingdom
+    >>> tldextract.extract("http://forums.bbc.co.uk/")  # United Kingdom
     ExtractResult(subdomain='forums', domain='bbc', suffix='co.uk', is_private=False)
 
-    >>> tldextract.extract('http://www.worldbank.org.kg/') # Kyrgyzstan
+    >>> tldextract.extract("http://www.worldbank.org.kg/")  # Kyrgyzstan
     ExtractResult(subdomain='www', domain='worldbank', suffix='org.kg', is_private=False)
 
 Note subdomain and suffix are _optional_. Not all URL-like inputs have a
 subdomain or a valid suffix.
 
-    >>> tldextract.extract('google.com')
+    >>> tldextract.extract("google.com")
     ExtractResult(subdomain='', domain='google', suffix='com', is_private=False)
 
-    >>> tldextract.extract('google.notavalidsuffix')
+    >>> tldextract.extract("google.notavalidsuffix")
     ExtractResult(subdomain='google', domain='notavalidsuffix', suffix='', is_private=False)
 
-    >>> tldextract.extract('http://127.0.0.1:8080/deployed/')
+    >>> tldextract.extract("http://127.0.0.1:8080/deployed/")
     ExtractResult(subdomain='', domain='127.0.0.1', suffix='', is_private=False)
 
 To rejoin the original hostname, if it was indeed a valid, registered hostname:
 
-    >>> ext = tldextract.extract('http://forums.bbc.co.uk')
+    >>> ext = tldextract.extract("http://forums.bbc.co.uk")
     >>> ext.registered_domain
     'bbc.co.uk'
     >>> ext.fqdn
@@ -36,7 +36,6 @@ To rejoin the original hostname, if it was indeed a valid, registered hostname:
 
 from __future__ import annotations
 
-import logging
 import os
 import urllib.parse
 from collections.abc import Collection, Sequence
@@ -50,9 +49,6 @@ from .cache import DiskCache, get_cache_dir
 from .remote import lenient_netloc, looks_like_ip, looks_like_ipv6
 from .suffix_list import get_suffix_lists
 
-LOG = logging.getLogger("tldextract")
-
-
 CACHE_TIMEOUT = os.environ.get("TLDEXTRACT_CACHE_TIMEOUT")
 
 PUBLIC_SUFFIX_LIST_URLS = (
@@ -65,21 +61,48 @@ PUBLIC_SUFFIX_LIST_URLS = (
 class ExtractResult:
     """A URL's extracted subdomain, domain, and suffix.
 
-    Also contains metadata, like a flag that indicates if the URL has a private suffix.
+    These first 3 fields are what most users of this library will care about.
+    They are the split, non-overlapping hostname components of the input URL.
+    They can be used to rebuild the original URL's hostname.
+
+    Beyond the first 3 fields, the class contains metadata fields, like a flag
+    that indicates if the input URL's suffix is from a private domain.
     """
 
     subdomain: str
+    """All subdomains beneath the domain of the input URL, if it contained any such subdomains, or else the empty string."""
+
     domain: str
+    """The topmost domain of the input URL, if it contained a domain name, or else everything hostname-like in the input.
+
+    If the input URL didn't contain a real domain name, the `suffix` field will
+    be empty, and this field will catch values like an IP address, or
+    private network hostnames like "localhost".
+    """
+
     suffix: str
+    """The public suffix of the input URL, if it contained one, or else the empty string.
+
+    If `include_psl_private_domains` was set to `False`, this field is the same
+    as `registry_suffix`, i.e. a domain under which people can register
+    subdomains through a registrar. If `include_psl_private_domains` was set to
+    `True`, this field may be a PSL private domain, like "blogspot.com".
+    """
+
     is_private: bool
+    """Whether the input URL belongs in the Public Suffix List's private domains.
+
+    If `include_psl_private_domains` was set to `False`, this field is always
+    `False`.
+    """
 
     @property
     def registered_domain(self) -> str:
-        """Joins the domain and suffix fields with a dot, if they're both set.
+        """The `domain` and `suffix` fields joined with a dot, if they're both set, or else the empty string.
 
-        >>> extract('http://forums.bbc.co.uk').registered_domain
+        >>> extract("http://forums.bbc.co.uk").registered_domain
         'bbc.co.uk'
-        >>> extract('http://localhost:8080').registered_domain
+        >>> extract("http://localhost:8080").registered_domain
         ''
         """
         if self.suffix and self.domain:
@@ -88,11 +111,11 @@ class ExtractResult:
 
     @property
     def fqdn(self) -> str:
-        """Returns a Fully Qualified Domain Name, if there is a proper domain/suffix.
+        """The Fully Qualified Domain Name (FQDN), if there is a proper `domain` and `suffix`, or else the empty string.
 
-        >>> extract('http://forums.bbc.co.uk/path/to/file').fqdn
+        >>> extract("http://forums.bbc.co.uk/path/to/file").fqdn
         'forums.bbc.co.uk'
-        >>> extract('http://localhost:8080').fqdn
+        >>> extract("http://localhost:8080").fqdn
         ''
         """
         if self.suffix and (self.domain or self.is_private):
@@ -101,13 +124,13 @@ class ExtractResult:
 
     @property
     def ipv4(self) -> str:
-        """Returns the ipv4 if that is what the presented domain/url is.
+        """The IPv4 address, if that is what the input domain/URL was, or else the empty string.
 
-        >>> extract('http://127.0.0.1/path/to/file').ipv4
+        >>> extract("http://127.0.0.1/path/to/file").ipv4
         '127.0.0.1'
-        >>> extract('http://127.0.0.1.1/path/to/file').ipv4
+        >>> extract("http://127.0.0.1.1/path/to/file").ipv4
         ''
-        >>> extract('http://256.1.1.1').ipv4
+        >>> extract("http://256.1.1.1").ipv4
         ''
         """
         if (
@@ -120,13 +143,17 @@ class ExtractResult:
 
     @property
     def ipv6(self) -> str:
-        """Returns the ipv6 if that is what the presented domain/url is.
+        """The IPv6 address, if that is what the input domain/URL was, or else the empty string.
 
-        >>> extract('http://[aBcD:ef01:2345:6789:aBcD:ef01:127.0.0.1]/path/to/file').ipv6
+        >>> extract(
+        ...     "http://[aBcD:ef01:2345:6789:aBcD:ef01:127.0.0.1]/path/to/file"
+        ... ).ipv6
         'aBcD:ef01:2345:6789:aBcD:ef01:127.0.0.1'
-        >>> extract('http://[aBcD:ef01:2345:6789:aBcD:ef01:127.0.0.1.1]/path/to/file').ipv6
+        >>> extract(
+        ...     "http://[aBcD:ef01:2345:6789:aBcD:ef01:127.0.0.1.1]/path/to/file"
+        ... ).ipv6
         ''
-        >>> extract('http://[aBcD:ef01:2345:6789:aBcD:ef01:256.0.0.1]').ipv6
+        >>> extract("http://[aBcD:ef01:2345:6789:aBcD:ef01:256.0.0.1]").ipv6
         ''
         """
         min_num_ipv6_chars = 4
@@ -141,6 +168,31 @@ class ExtractResult:
                 return debracketed
         return ""
 
+    @property
+    def reverse_domain_name(self) -> str:
+        """The domain name in Reverse Domain Name Notation.
+
+        Joins extracted components of the input URL in reverse domain name
+        notation. The suffix is used as the leftmost component, followed by the
+        domain, then followed by the subdomain with its parts reversed.
+
+        Reverse Domain Name Notation is typically used to organize namespaces
+        for packages and plugins. Technically, a full reversal would reverse
+        the parts of the suffix, e.g. "co.uk" would become "uk.co", but this is
+        not done in practice when Reverse Domain Name Notation is called for.
+        So this property leaves the `suffix` part in its original order.
+
+        >>> extract("login.example.com").reverse_domain_name
+        'com.example.login'
+
+        >>> extract("login.example.co.uk").reverse_domain_name
+        'co.uk.example.login'
+        """
+        stack = [self.suffix, self.domain]
+        if self.subdomain:
+            stack.extend(reversed(self.subdomain.split(".")))
+        return ".".join(stack)
+
 
 class TLDExtract:
     """A callable for extracting, subdomain, domain, and suffix components from a URL."""
@@ -161,13 +213,14 @@ class TLDExtract:
         the `cache_dir` will live in the tldextract directory. You can disable
         the caching functionality of this module by setting `cache_dir` to `None`.
 
-        If the cached version does not exist (such as on the first run), HTTP request the URLs in
-        `suffix_list_urls` in order, until one returns public suffix list data. To disable HTTP
-        requests, set this to an empty sequence.
-
-        The default list of URLs point to the latest version of the Mozilla Public Suffix List and
-        its mirror, but any similar document could be specified. Local files can be specified by
-        using the `file://` protocol. (See `urllib2` documentation.)
+        If the cached version does not exist, such as on the first run, HTTP
+        request the URLs in `suffix_list_urls` in order, and use the first
+        successful response for public suffix definitions. Subsequent, untried
+        URLs are ignored. The default URLs are the latest version of the
+        Mozilla Public Suffix List and its mirror, but any similar document URL
+        could be specified. Local files can be specified by using the `file://`
+        protocol (see `urllib2` documentation). To disable HTTP requests, set
+        this to an empty sequence.
 
         If there is no cached version loaded and no data is found from the `suffix_list_urls`,
         the module will fall back to the included TLD set snapshot. If you do not want
@@ -179,7 +232,9 @@ class TLDExtract:
         suffix, so these domains are excluded by default. If you'd like them
         included instead, set `include_psl_private_domains` to True.
 
-        You can pass additional suffixes in `extra_suffixes` argument without changing list URL
+        You can specify additional suffixes in the `extra_suffixes` argument.
+        These will be merged into whatever public suffix definitions are
+        already in use by `tldextract`, above.
 
         cache_fetch_timeout is passed unmodified to the underlying request object
         per the requests documentation here:
@@ -237,9 +292,9 @@ class TLDExtract:
         I.e. its effective TLD, gTLD, ccTLD, etc. components.
 
         >>> extractor = TLDExtract()
-        >>> extractor.extract_str('http://forums.news.cnn.com/')
+        >>> extractor.extract_str("http://forums.news.cnn.com/")
         ExtractResult(subdomain='forums.news', domain='cnn', suffix='com', is_private=False)
-        >>> extractor.extract_str('http://forums.bbc.co.uk/')
+        >>> extractor.extract_str("http://forums.bbc.co.uk/")
         ExtractResult(subdomain='forums', domain='bbc', suffix='co.uk', is_private=False)
 
         Allows configuring the HTTP request via the optional `session`
@@ -272,9 +327,11 @@ class TLDExtract:
         name has already been parsed.
 
         >>> extractor = TLDExtract()
-        >>> extractor.extract_urllib(urllib.parse.urlsplit('http://forums.news.cnn.com/'))
+        >>> extractor.extract_urllib(
+        ...     urllib.parse.urlsplit("http://forums.news.cnn.com/")
+        ... )
         ExtractResult(subdomain='forums.news', domain='cnn', suffix='com', is_private=False)
-        >>> extractor.extract_urllib(urllib.parse.urlsplit('http://forums.bbc.co.uk/'))
+        >>> extractor.extract_urllib(urllib.parse.urlsplit("http://forums.bbc.co.uk/"))
         ExtractResult(subdomain='forums', domain='bbc', suffix='co.uk', is_private=False)
         """
         return self._extract_netloc(
@@ -298,9 +355,9 @@ class TLDExtract:
             len(netloc_with_ascii_dots) >= min_num_ipv6_chars
             and netloc_with_ascii_dots[0] == "["
             and netloc_with_ascii_dots[-1] == "]"
+            and looks_like_ipv6(netloc_with_ascii_dots[1:-1])
         ):
-            if looks_like_ipv6(netloc_with_ascii_dots[1:-1]):
-                return ExtractResult("", netloc_with_ascii_dots, "", is_private=False)
+            return ExtractResult("", netloc_with_ascii_dots, "", is_private=False)
 
         labels = netloc_with_ascii_dots.split(".")
 
@@ -330,9 +387,9 @@ class TLDExtract:
 
     @property
     def tlds(self, session: requests.Session | None = None) -> list[str]:
-        """Returns the list of tld's used by default.
+        """The list of TLDs used by default.
 
-        This will vary based on `include_psl_private_domains` and `extra_suffixes`
+        This will vary based on `include_psl_private_domains` and `extra_suffixes`.
         """
         return list(self._get_tld_extractor(session=session).tlds())
 
